﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Emgu.CV;
using Emgu.CV.Structure;
using bbiwarg.Detectors.Fingers;

using System.Diagnostics;
using bbiwarg.Utility;
using bbiwarg.Graphics;

namespace bbiwarg.Images
{
    class EdgeImage
    {
        public Image<Gray, Byte> Image { get; private set; }

        public EdgeImage(DepthImage depthImage, OutputImage outputImage)
        {
            Image<Gray, byte> dimg = depthImage.Image;
            //rescale depthImage to [0,255] -> performance loss not worth (is already scaled to [0,200])
            /*double[] min, max;
            Point[] minLoc, maxLoc;
            dimg.MinMax(out min, out max, out minLoc, out maxLoc);
            double interval = max[0] - min[0];
            dimg = dimg.Convert(delegate(byte d) {return (byte) ((d/interval)*255);});*/
            
            Image = dimg.Canny(150, 100, 3);

            // draw blue edges in outputImage
            outputImage.Image[2] = Image.ThresholdBinary(new Gray(0), new Gray(1)).Mul(255);
        }

        public EdgeImage(Image<Gray, Byte> edgeImage)
        {
            Image = edgeImage;
        }

        public bool isEdgeAt(Point point)
        {
            return isEdgeAt(point.X, point.Y);
        }

        public bool isEdgeAt(int x, int y)
        {
            return (Image.Data[y, x, 0] > 0);
        }

        public void removeFingerEdges(Finger finger)
        {
            Point[] polygon = finger.getPolygon();

            Image.FillConvexPoly(polygon, new Gray(0));
        }

        public EdgeImage copy()
        {
            return new EdgeImage(Image.Copy());
        }
    }
}
